# coding: utf-8
from caty.core.command import Builtin
from caty.core.exception import *
from caty.command.file import FileUtilMixin
import caty.jsontools as json
import caty.jsontools.xjson as xjson

name = 'xjson'
schema = u"""
type AnyObject = {*:any};
"""

class ReadJson(FileUtilMixin, Builtin):
    command_decl = u"""
        /**
         * 引数で指定されたファイルを JSON データとして読み込み、値を返す。
         * JSON でないデータの書き込まれたファイルに対しての動作は保証しない（通常はエラー）。
         */
        command read [string] :: void -> any
            reads [pub, data]
            refers python:caty.command.xjsonlib.ReadJson;
    """
    def execute(self):
        f = self.open('rb')
        if not f.exists:
            raise FileNotFound(f.path)
        j = xjson.load(f)
        f.close()
        return j

class WriteJson(FileUtilMixin, Builtin):
    command_decl = u"""
        /**
         * 引数で指定されたファイルに JSON データを書き込む。
         * このコマンドで書き込んだデータは json:read で読み出し可能である。
         */
        command write [string] :: any -> void
            updates [data, pub]
            refers python:caty.command.xjsonlib.WriteJson;
    """

    def execute(self, input):
        f = self.open('wb')
        xjson.dump(input, f)
        f.close()

