from __future__ import with_statement
from threading import Thread, RLock
import time
import types

class AsyncQueue(object):
    def __init__(self, app):
        self._app = app
        self._queue = {}
    
    def push(self, callee, func, *args, **kwds):
        t = AsyncThread(self, callee, func, *args, **kwds)
        t.start()
        with RLock():
            self._queue[t.getName()] = t

    def remove(self, t):
        with RLock():
            del self._queue[t.getName()]
    
class AsyncThread(Thread):
    def __init__(self, creator, callee, func, *args, **kwds):
        Thread.__init__(self)
        self._creator = creator
        self._func = func
        self._args = args
        self._kwds = kwds
        f = callee._new_facility()
        self._obj = callee.clone()
        self._obj.set_facility(f)

    def run(self):
        try:
            time.sleep(1)
            if isinstance(self._func, types.MethodType) and type(self._func.im_self) == type(self._obj):
                self._func.im_func(self._obj, *self._args, **self._kwds)
            else:
                self._func(self._obj, *self._args, **self._kwds)
            self._obj._internal_commit()
        finally:
            self._creator.remove(self)

