#coding:utf-8
u"""
"""

from topdown import *
from caty.core.casm.language.ast import *
from caty.core.casm.language.util import *
from caty.core.schema import schemata

RESERVED = frozenset(schemata.keys())

def schema(seq):
    doc = option(docstring)(seq)
    a = seq.parse(annotation)
    t = seq.parse('type ')
    n = seq.parse(name)
    if n in RESERVED:
        raise ParseFailed('%s is reserved.' % n)
    v = seq.parse(option(type_arg))
    e = seq.parse('=')
    d = seq.parse(typedef)
    if d.optional:
        raise ParseFailed(seq, schema)
    c = seq.parse(';')
    return TypeReference(n, v, d, a)

def typedef(seq):
    return chainl(term, op)(seq)

def op(seq):
    o = seq.parse(['&', '|', '<<'])
    if o == '&':
        return IntersectionNode
    elif o == '<<':
        return UpdatorNode
    else:
        return UnionNode

def term(seq):
    def _tag(s):
        _ = s.parse('@')
        t = s.parse([tagname, string, '*'])
        v = s.parse(option(term)) # @foo だけなら @foo any (or @foo _T)?
        return TaggedNode(t, v)
    def _pseudo_tag(s):
        s.parse('@?')
        s.parse('(')
        n = seq.parse(Regex(u'(([a-zA-Z_][a-zA-Z0-9_]*)|\$)'))
        s.parse('=')
        v = s.parse(value)
        s.parse(')')
        t = s.parse(term)
        return PseudoTaggedNode(n, v, t)
    def _term(s):
        _ = s.parse('(')
        d = s.parse(typedef)
        _ = s.parse(')')
        return d
    def _never(s):
        _ = s.parse(Regex(r'\(\s*\)'))
        return ScalarNode('never', {}, [])
    s = seq.parse([_pseudo_tag, _tag, enum, _never, _term, object_, array, scalar])
    o = seq.parse(option('?'))
    if o:
        s.optional = True
    return s

def tagname(seq):
    return seq.parse(Regex(u'[a-zA-Z0-9_][a-zA-Z0-9-_]*'))

def type_arg(seq):
    _ = seq.parse('<')
    vars = chain_flat(typename, comma)(seq)
    _ = seq.parse('>')
    return vars

@try_
def type_var(seq):
    _ = seq.parse('<')
    vars = chain_flat(typedef, comma)(seq)
    _ = seq.parse('>')
    return vars

def scalar(seq):
    n = seq.parse(typename)
    o = seq.parse(option(options, {}))
    t = seq.parse(option(type_var, []))
    node = ScalarNode(n, o, t)
    return node

def typename(seq):
    app = seq.parse(option(Regex(r'[a-zA-Z_][a-zA-Z0-9_]*:'), ''))
    module = seq.parse(option(Regex(r'(([a-zA-Z_][a-zA-Z0-9_]*\.)*([a-zA-Z][a-zA-Z0-9]*:))?'), ''))
    name = seq.parse(Regex(u'(([a-zA-Z_][a-zA-Z0-9_]*)|\$)'))
    return ''.join([app, module, name])
    #return seq.parse(Regex(r'(([a-zA-Z_][a-zA-Z0-9_]*#)?(\.[_a-zA-Z][_a-zA-Z0-9]*)*\:)?(([_a-zA-Z][_a-zA-Z0-9]*)|(\$))'))

def options(seq):
    _ = seq.parse('(')
    opts = option(chain_flat(optval, comma))(seq)
    _ = seq.parse(')')
    if not opts: return {}
    return dict(opts)

def optval(seq):
    k = seq.parse(name)
    _ = seq.parse('=')
    v = seq.parse(value)
    return [k, v]

def array(seq):
    seq.parse('[')
    r = option(chain_flat(repeatable_type, comma), [])(seq)
    seq.parse(']')
    o = seq.parse(option(options, {}))
    o['repeat'] = False
    l = []
    for s, opt in r:
        if opt == '?':
            s.optional = True
        elif opt == '*':
            o['repeat'] = True
        l.append(s)
    a = ArrayNode(l, o)
    return a

def repeatable_type(seq):
    s = seq.parse(term)
    r = seq.parse(option(['*', '?']))
    return [s, r]

def maybe_tagged_value(seq):
    def _tagged(s):
        _ = s.parse('@')
        t = s.parse([name, '*'])
        v = s.parse(value)
        return TaggedNode(t, v)
    return seq.parse([_tagged, value])

def enum(seq):
    #values = seq.parse(chain_flat(maybe_tagged_value, '|'))
    v = [maybe_tagged_value(seq)]
    return EnumNode(v)

def object_(seq):
    _ = seq.parse(option('{'))
    items = {}
    ls = option(chain_flat(item, comma), [])(seq)
    for k, v in ls:
        if k in items:
            raise KeyError(k)
        items[k] = v
    _ = seq.parse('}')
    o = seq.parse(option(options, {}))
    w = None
    if '*' in items:
        w = items['*']
        del items['*']
    return ObjectNode(items, w, o)

def item(seq):
    k = seq.parse([string, '*'])
    _ = seq.parse(':')
    s = typedef(seq)
    return [k, s]

