# coding:utf-8
from topdown import *
from decimal import Decimal
import re

#__all__ = ['annotation', 'name', 'comma', 'chain_flat', 'docstring', 'SchemaSyntaxError', 'remove_comment']

def name(seq):
    return seq.parse(Regex(r'(([a-zA-Z_][a-zA-Z0-9_]*)|(\$))'))

def comma(seq):
    _ = seq.parse(',')
    return _

class chain_flat(Parser):
    def __init__(self, parser, op):
        self.parser = parser
        self.op = op

    def __call__(self, seq):
        r = []
        r.append(seq.parse(self.parser))
        while True:
            try:
                a = seq.parse(self.op)
                b = seq.parse(self.parser)
            except ParseFailed, e:
                break
            else:
                #カンマへの対応のみなので、 op の結果は捨てる
                r.append(b)
        return r

def docstring(seq):
    seq.parse(skip_ws)
    seq.parse('/**')
    doc = seq.parse(until('*/')).strip('\n')#Regex(r'/\*\*.+?\*/', re.S))[3:-2].strip('\n')
    seq.parse('*/')
    seq.parse(skip_ws)
    r = []
    lines = [d.strip() for d in doc.splitlines()]
    space = 0
    for i in list(lines[0]):
        if i == ' ':
            space += 1
        else:
            break
    d = []
    for l in lines:
        l = l.replace(' '*space, '', 1)
        if l == '*':
            l = ''
        elif l.startswith('* '):
            l = l[2:]
        d.append(l)
    return u'\n'.join(d)

class SchemaSyntaxError(Exception):
    def __init__(self, exception):
        self.exception = exception

    def __str__(self):
        return str(self.exception)

    def __repr__(self):
        return repr(self.exception)

def value(seq):
    return seq.parse([string, boolean, number, integer])

def boolean(seq):
    return bool(seq.parse(['true', 'false']).title())

def integer(seq):
    return int(seq.parse(Regex(r'-?[0-9]+')))

def number(seq):
    return Decimal(seq.parse(Regex(r'-?[0-9]+\.[0-9]+')))


def string(seq):
    _ = seq.parse('"')
    s = seq.parse(until('"'))
    if '\r' in s or '\n' in s:
        raise ParseFailed(seq, string)
    _ = seq.parse('"')
    return s


def annotation(seq):
    a = option(Regex(r'@\[([a-zA-Z][-a-zA-Z0-9]*[, ]*)*\]+'))(seq)
    if a:
        return a.strip()[2:-1].strip() # @[]を取り除く
    return a

def remove_comment(seq):
    many(_comment)(seq)

def _comment(seq):
    doc_str = seq.peek(_is_doc_str)
    if doc_str:
        raise ParseFailed(seq, _comment)
    s = seq.parse(['//', '/*'])
    if s == '//':
        seq.parse([until('\n'), until('\r')])
        seq.parse(skip_ws)
    elif s == '/*':
        seq.parse(until('*/'))
        seq.parse('*/')
        seq.parse(skip_ws)

def _is_doc_str(seq):
    _ = seq.parse(option(docstring))
    if _:
        try:
            seq.parse(skip_ws)
            seq.parse(annotation)
            seq.parse(skip_ws)
            seq.parse(['command', 'type'])
            return True
        except:
            return False
    return False

