#coding: utf-8
import caty.jsontools as json
class CatyException(Exception):
    u"""Caty 例外。詳細は specdocs 参照
    """
    def __init__(self, error_tag, message, error_class=None, error_id=None, stack_trace=None):
        o = {}
        o['message'] = message
        if error_class:
            o['class'] = error_class
        if error_id:
            o['id'] = error_id
        if stack_trace:
            o['stackTrace'] = stack_trace
        self.__json_obj = json.tagged(error_tag, o)
        Exception.__init__(self, self.tag)

    @property
    def tag(self):
        return json.tag(self.__json_obj)

    @property
    def error_obj(self):
        return json.untagged(self.__json_obj)

    def to_json(self):
        return self.__json_obj

    def get_message(self, lang):
        if lang in self.error_obj:
            return self.error_obj['message'].get(lang)
        else:
            return self.error_obj['message'].get('en')

    def __unicode__(self):
        return self.get_message('en')

def throw_caty_exception(tag, message, error_class=None, error_id=None, stack_trace=None):
    raise CatyException(tag, message, error_class, error_id, stack_trace)

class SubCatyException(CatyException):
    def __init__(self, place_holder):
        CatyException.__init__(self, self.error_type, self._apply(self.message_map, place_holder))

    def _apply(self, d, m):
        r = {}
        for k, v in d.items():
            r[k] = v % m
        return r


class ScriptNotFound(SubCatyException):
    error_type = 'ScriptNotFound'
    message_map = {
        'ja': u'スクリプトファイルが存在しません: %s',
        'en': u'Script file does not exists: %s'
    }

class FileNotFound(SubCatyException):
    error_type = 'FileNotFound'
    message_map = {
        'ja': u'ファイルが存在しません: %s',
        'en': u'File does not exists: %s'
    }


class FileIOError(SubCatyException):
    error_type = 'FileIOError'
    message_map = {
        'ja': u'ファイルの読み込みに失敗しました: %s',
        'en': u'Failed to read file: %s'
    }

class TableNotFound(SubCatyException):
    error_type = 'TableNotFound'
    message_map = {
        'ja': u'テーブルが存在しません: %s',
        'en': u'Table does not exists: %s'
    }


class TableAccessError(SubCatyException):
    error_type = 'TableAccessError'
    message_map = {
        'ja': u'テーブルへのアクセスに失敗しました: %s',
        'en': u'Failed to read table: %s'
    }




