#coding: utf-8
from caty.core.schema.base import *

class ArraySchema(SchemaBase):
    def __init__(self, schema_list, *args, **kwds):
        SchemaBase.__init__(self, *args, **kwds)
        self.schema_list = schema_list
    
    maxItems = attribute('maxItems')
    minItems = attribute('minItems')
    repeat = attribute('repeat')

    def _validate(self, value):
        if not self.optional and value == None:
            raise JsonSchemaError(u'null は許可されていません')
        elif self.optional and value is None:
            return
        if not (isinstance(value, list) or isinstance(value, tuple)):
            raise JsonSchemaError(u'値はリストでなければなりません')
        if self.maxItems is not None and len(value) > self.maxItems:
            raise JsonSchemaError(u'この型には %d 個以内の要素数しか挿入できません' % (len(self.schema_list)))
        count = 0
        if self.minItems is not None and len(value) < self.minItems:
            raise JsonSchemaError(u'この型には %d 個以上の要素を挿入しなければなりません' % (self.minItems))
        mandatory = len(self.schema_list)
        if self.repeat:
            mandatory -= 1
        for s in self.schema_list:
            if s.optional:
                mandatory -= 1
        l = len(value)
        if l < mandatory:
            raise JsonSchemaError(u'要素数が足りません: %d 個の要素が渡されましたが %d 個必要です' % (l, mandatory))
        if l > len(self.schema_list) and not self.repeat:
            raise JsonSchemaError(u'要素数が多すぎます: %d 個の要素が渡されましたが %d 個が上限です' % (l, len(self.schema_list)))
        errors = []
        is_error = False
        for v in value:
            if count >= len(self.schema_list):
                if not self.repeat:
                    raise JsonSchemaError(u'この型には %d 個以内の要素数しか挿入できません' % (len(self.schema_list)))
                elif self.repeat:
                    count = len(self.schema_list) - 1
            try:
                self.schema_list[count].validate(v)
                errors.append(ErrorObj(False, v, v, ''))
            except JsonSchemaError, e:
                errors.append(e)
                is_error = True
            count += 1
        if is_error:
            e = JsonSchemaErrorList(u'array 型の検証に失敗しました', errors)
            raise e

    def _convert(self, value):
        if not (isinstance(value, list) or isinstance(value, tuple)):
            if value is None and self.schema_list[0].optional:
                return []
            raise JsonSchemaError(u'値はリストでなければなりません')
        count = 0
        result =[]
        errors = []
        is_error = False
        for v in value:
            if count >= len(self.schema_list):
                if not self.repeat:
                    raise JsonSchemaError(u'この型には %d 個以内の要素数しか挿入できません' % (len(self.schema_list)))
                elif self.repeat:
                    count = len(self.schema_list) - 1
            try:
                result.append(self.schema_list[count].convert(v))
                errors.append(ErrorObj(False, v, result[-1], ''))
            except JsonSchemaError, e:
                is_error = True
                errors.append(e)
            count += 1
        if is_error:
            e = JsonSchemaErrorList(u'array 型の検証に失敗しました', errors)
            raise e
        return result

    def _deepcopy(self):
        return ArraySchema(self.schema_list, self.options)

    def _clone(self, options, schema_list=None):
        if options is not None:
            if self.repeat is not None and 'repeat' not in options:
                options['repeat'] = self.repeat
            if self.minItems is not None and 'minItems' not in options:
                options['minItems'] = self.minItems
            if self.maxItems is not None and 'maxItems' not in options:
                options['maxItems'] = self.maxItems

            return ArraySchema(schema_list if schema_list is not None else self.schema_list, 
                               options)
        else:
            return ArraySchema(schema_list if schema_list is not None else self.schema_list, 
                               self.options)

    def dump(self, depth, node=[]):
        r = []
        for s in self.schema_list:
            r.append(s.dump(depth, node))
        if self.repeat:
            r[-1] += '*'
        return '[%s]' % ', '.join(r)

    def intersect(self, another):
        al = another.schema_list
        r = []
        for a, b in zip(self.schema_list, al):
            r.append(a & b)
        o = {}
        if self.maxItems and another.maxItems:
            o['maxItems'] = min(self.maxItems, another.maxItems)
        else:
            o['maxItems'] = self.maxItems if self.maxItems != None else another.maxItems
        if self.minItems and another.minItems:
            o['minItems'] = max(self.minItems, another.minItems)
        else:
            o['minItems'] = self.minItems if self.minItems != None else another.minItems
        if self.optional or another.optional:
            o['optional'] = True
        if self.repeat and another.repeat:
            o['repeat'] = True
        n = ArraySchema(r, o)
        return n
       

    @property
    def type(self):
        return 'array'

    def apply(self, vars):
        l = []
        for s in self.schema_list:
            if s.type == '__variable__' and s.name in vars:
                l.append(s.apply(vars))
            else:
                l.append(s)
        return ArraySchema(l, self._options)

    @property
    def type_vars(self):
        v = []
        for s in self.schema_list:
            for t in s.type_vars:
                if t not in v:
                    v.append(t)
        return v


