#coding: utf-8
from caty.core.schema.base import *

class BinarySchema(ScalarSchema):

    minlength = attribute('minLength')
    maxlength = attribute('maxLength')

    def _validate(self, value):
        if not self.optional and value == None:
            raise JsonSchemaError(u'null は許可されていません')
        elif self.optional and value == None:
            return
        if not isinstance(value, str):
            raise JsonSchemaError(u'値はバイト列でなければなりません')
        if self.minlength != None and self.minlength > len(value):
            raise JsonSchemaError(u'データ長は %d よりも大きくなければなりません' % self.minlength)
        if self.maxlength != None and self.maxlength < len(value):
            raise JsonSchemaError(u'データ長は %d よりも小さくなければなりません' % self.maxlength)

    def intersect(self, another):
        minlength = max(self.minlength, another.minlength)
        if None in (self.maxlength, another.maxlength):
            maxmum = self.maxlength if another.maxlength is None else another.maxlength
        else:
            maxmum = min(self.maxlength, another.maxlength)
        if minlength > maxlength and maxlength is not None:
            raise JsonSchemaError(u'& 演算の結果が不正です: minlength よりも小さな maxlength が渡されました')
        opts = {
            'minlength': minlength,
            'maxlength': maxlength,
        }
        return self.clone(opts, self.module)
 
    def _convert(self, value):
        if isinstance(value, str):
            return value
        return str(value)

    def dump(self, depth, node=[]):
        return 'binary'

    @property
    def type(self):
        return 'binary'


