#coding: utf-8
from caty.core.schema.base import *
from caty.util import try_parse
from decimal import Decimal

class EnumSchema(SchemaBase):
    def __init__(self, enum, *args, **kwds):
        SchemaBase.__init__(self, *args, **kwds)
        self.enum = enum

    def _validate(self, value):
        if not self.optional and value is None:
            raise JsonSchemaError(u'null は許可されていません', value, '')
        elif self.optional and value is None:
            return
        for e in self.enum:
            if isinstance(e, SchemaBase):
                try:
                    e.validate(value)
                except:
                    pass
                else:
                    return
            else:
                if e == value:
                    if (type(e) == type(value)) or (not isinstance(value, bool) and isinstance(value, (int, Decimal)) and isinstance(e, (int, Decimal)) and isinstance(value, (int, Decimal))):
                        return
        raise JsonSchemaError(u'値は次のいずれかでなくてはなりません: %s' % self._to_str(), value, '')

    def union(self, another):
        if isinstance(another, EnumSchema):
            return EnumSchema(self.enum + another.enum, self.options)
        else:
            return SchemaBase.union(self, another)

    def intersect(self, another):
        en = []
        if another.type == 'enum':
            for e in another.enum:
                if e in self.enum:
                    en.append(e)
            return EnumSchema(en, self.options)
        else:
            raise JsonSchemaError(u'enum と %s の & 演算は定義されていません' % (another.type))

    def _deepcopy(self):
        return EnumSchema(self.enum, self.options)

    def _clone(self, enum=None, options=None):
        return EnumSchema(enum if enum else self.enum, 
                          options if options else self.options)

    def _convert(self, value):
        for e in self.enum:
            if isinstance(e, unicode):
                if value == e:
                    return e
            else:
                if try_parse(int, value) == e:
                    return e
        raise JsonSchemaError(u'値は次のいずれかでなくてはなりません: %s' % self._to_str(), value, '')

    def apply(self, vars):
        l = []
        for s in self.enum:
            if isinstance(s, SchemaBase) and s.type == '__variable__' and s.name in vars:
                l.append(s.apply(vars))
            else:
                l.append(s)
        return EnumSchema(l, self._options)


    def _to_str(self):
        r = []
        for e in self.enum:
            if isinstance(e, unicode):
                r.append('"%s"' % e)
            else:
                r.append(str(e))
        return ', '.join(r)

    def dump(self, depth, node=[]):
        r = []
        for e in self.enum:
            if isinstance(e, unicode):
                r.append('"%s"' % e)
            else:
                r.append(str(e))
        return ' | '.join(r)

    @property
    def type(self):
        return 'enum'

