#coding: utf-8
from caty.core.schema.base import *
from decimal import Decimal
class NumberSchema(ScalarSchema):
    minimum = attribute('minimum')
    maximum = attribute('maximum')
    is_integer = False

    def _validate(self, value):
        if not self.optional and value == None:
            raise JsonSchemaError(u'null は許可されていません')
        elif self.optional and value == None:
            return
        if self.is_integer:
            if not isinstance(value, int) or isinstance(value, bool):
                raise JsonSchemaError(u'%s は整数ではありません' % str(value), value, '')
        if not isinstance(value, (Decimal, int)):
            raise JsonSchemaError(u'%s は数値ではありません' % str(value), value, '')
        if self.minimum != None and self.minimum > value:
            raise JsonSchemaError(u'値は %f 未満でなければなりません' % self.minimum, value, '')
        if self.maximum != None and self.maximum < value:
            raise JsonSchemaError(u'値は %f よりも大きくなければなりません' % self.maximum, value, '')

    def __and__(self, another):
        minimum = max(self.minimum, another.minimum)
        if None in (self.maximum, another.maximum):
            maxmum = self.maximum if another.maximum is None else another.maximum
        else:
            maxmum = min(self.maximum, another.maximum)
        is_integer = self.is_integer or another.is_integer
        if minimum > maximum and maximum is not None:
            raise JsonSchemaError(u'& 演算の結果が不正です: minimum よりも小さな maximum が渡されました')
        opts = {
            'minimum': minimum,
            'maximum': maximum,
            'is_integer': is_integer
        }
        return self.clone(opts, self.module)

    def _convert(self, value):
        try:
            if self.is_integer:
                return int(value)
            return Decimal(value)
        except:
            raise JsonSchemaError(u'数値型への変換に失敗しました', value, '')

    def dump(self, depth, node=[]):
        if self.is_integer:
            return 'integer'
        return 'number'

    @property
    def type(self):
        if self.is_integer:
            return 'integer'
        return 'number'

class IntegerSchema(NumberSchema):
    is_integer = True

