#coding: utf-8
from caty.core.schema.base import *

class StringSchema(ScalarSchema):

    minLength = attribute('minLength')
    maxLength = attribute('maxLength')

    def _validate(self, value):
        if not self.optional and value == None:
            raise JsonSchemaError(u'null は許可されていません')
        elif self.optional and value == None:
            return
        if not isinstance(value, unicode):
            raise JsonSchemaError(u'値は unicode でなければなりません')
        if self.minLength != None and self.minLength > len(value):
            raise JsonSchemaError(u'文字列長は %d よりも長くなければなりません' % self.minLength)
        if self.maxLength != None and self.maxLength < len(value):
            raise JsonSchemaError(u'文字列長は %d よりも短くなければなりません' % self.maxLength)

    def intersect(self, another):
        minLength = max(self.minLength, another.minLength)
        maxLength = min(self.maxLength, another.maxLength)
        if None in (self.maxLength, another.maxLength):
            maxmum = self.maxLength if another.maxLength is None else another.maxLength
        else:
            maxmum = min(self.maxLength, another.maxLength)
        if minLength > maxLength and maxLength is not None:
            raise JsonSchemaError(u'& 演算の結果が不正です: minLength よりも小さな maxLength が渡されました')
        opts = {
            'minLength': minLength,
            'maxLength': maxLength,
        }
        return self.clone(opts)
        

    def _convert(self, value):
        if value == None or isinstance(value, unicode):
            return value
        elif isinstance(value, basestring):
            return unicode(value)
        else:
            raise JsonSchemaError(u'文字列のみが変換対象です', unicode(repr(value)), '')

    def dump(self, depth, node=[]):
        return 'string'

    @property
    def type(self):
        return 'string'


