# coding:utf-8
from caty.core.command import Builtin, Command

class CommandBuilder(object):
    def __init__(self, facilities, namespace):
        self.facilities = facilities
        self.namespace = namespace

    def build(self, name, opts, args):
        u"""コマンド文字のチャンクをコマンド名と引数のリストに分割し、呼び出し可能なコマンドオブジェクトを返す。
        """
        cls = self.namespace[name]
        cmd = cls(opts, args)
        cmd.set_facility(self.facilities)
        cmd._prepare()
        return cmd

    def set_facility_to(self, c):
        u"""Syntax オブジェクトなど、通常の処理経路をたどらないオブジェクトにファシリティを追加する。
        """
        c.set_facility(self.facilities)


class CommandCombinator(Command):
    u"""二つのコマンドを受け取り、それらを順次実行するコマンドのラッパークラス。
    """
    def __init__(self, bf, af):
        self.bf = bf
        self.af = af
    
    def convert(self, value):
        return self.bf.convert(value)

    def set_facility(self, facilities):
        self.bf.set_facility(facilities)
        self.af.set_facility(facilities)

    @property
    def in_schema(self):
        return self.bf.in_schema

    @property
    def out_schema(self):
        return self.af.out_schema

    def __call__(self, input):
        r = self.bf(input)
        return self.af(r)

def combine(chunks):
    return reduce(CommandCombinator, chunks)

