#coding:utf-8
from getopt import getopt
from caty.core.application import System, StdStream, StreamWrapper
from caty.core.script.builder import CommandCombinator
from caty.core.script.parser import NothingTodo
from caty.jsontools import pp
from caty.core.schema import VoidSchema, ArraySchema
from caty.mafs.authorization import AuthoriToken
from caty.session.value import create_variable
from caty.util import get_message, init_writer, cout, debug
import caty

import cmd, glob, os
import locale
import traceback

def get_encoding():
    try:
        l = locale.getpreferredencoding()
        try:
            ''.decode(l)
            return l
        except Exception, e:
            init_writer('utf-8')
            debug.write(e)
            return 'utf-8'
    except Exception, e:
        init_writer('utf-8')
        debug.write(e)
        return 'utf-8'

def catch(f):
    def _(*args, **kwds):
        try:
            return f(*args, **kwds)
        except Exception, e:
            cout.write(traceback)
            cout.write(e)
            return None
    _.__doc__ = f.__doc__
    return _

class CatyShell(cmd.Cmd):
    def __init__(self, site, wildcat, debug, system):
        cmd.Cmd.__init__(self)
        self.debug = debug
        self.intro = """Caty interactive shell
        """
        stream = StreamWrapper(StdStream(site.sysencoding))
        self.stream = stream.dual_mode
        self.app = site
        self.interpreter = None
        self.wildcat = wildcat
        self.set_prompt()
        self.system = system
        self.last_session = None
        self.server = None
        self.system = system

    def start_pipeline(self):
        facilities = self.app.create_facilities(self.last_session)
        if self.interpreter is None:
            self.interpreter = self.app.interpreter.shell_mode(facilities)
        else:
            self.interpreter.update_facility(facilities)
        self.file_loader = self.app.interpreter.file_mode(facilities)
        if not self.wildcat:
            modes = [unicode('console')]
        else:
            modes = [unicode('console'), unicode('test')]
        self.app.init_env(facilities, self.debug, modes, self.system)
        return facilities

    def change_app(self, line):
        if not line.strip():
            self._echo('Usage: change <APP NAME>')
            return False
        name = line.strip()
        if name in self.system.app_names:
            self.app = self.system.get_app(name)
            self.set_prompt()
            self.interpreter = None
        else:
            self._echo(u'%s というアプリケーションは存在しないか、起動しない設定になっています' % line.strip())
        return False

    @catch
    def do_help(self, line):
        u"""
Usage: help [オプション] <モジュール名:コマンド名>

    *               ビルトインモジュールのコマンド一覧を表示
    command         ビルトインモジュールの command のヘルプを表示
    *:              モジュール一覧を表示
    module:*        module のコマンド一覧を表示
    module:command  module:command のヘルプを表示

コマンドのヘルプを表示する。
コマンド名が省略された場合、一覧表示になる。

特殊コマンド:
    change
    help
    quit
    reload
    server
"""
        echo = self._echo
        facilities = self.app.create_facilities(self.last_session)
        interpreter = facilities['interpreter']
        line = line.strip()
        if line == '':
            line = 'help'
        if line in ('change', 'reload', 'l', 'quit', 'help'):
            h = getattr(self, 'do_%s' % line)
            echo(h.__doc__)
            return
        module = 'builtin'
        mode = 'usage' # usage or list or module_usage oe list_modules
        command = ''
        chunk = line.split(':')
        if len(chunk) == 1:
            if chunk[0] != '*':
                command = chunk[0]
                mode = 'usage'
            else:
                mode = 'list'
        elif len(chunk) == 2:
            if chunk[0] == '*':
                if chunk[1] == '':
                    mode = 'list_modules'
                else:
                    echo(u'引数エラー: %s' % line)
                    return 
            else:
                module = chunk[0]
                if chunk[1] == '':
                    echo(u'引数エラー: %s' % line)
                    return 
                elif chunk[1] == '*':
                    mode = 'list'
                else:
                    command = chunk[1]
        if mode == 'usage':
            if interpreter.has_command(module, command):
                echo(interpreter.get_help(module, command))
            else:
                echo(u'未知のコマンド: %s' % command)
        elif mode == 'list':
            commands = list(interpreter.get_commands(module))
            commands.sort(cmp=lambda x, y:cmp(x[0], y[0]))
            max_width = max(map(lambda a:len(a[0]), commands))
            echo(u'モジュール: %s' % module)
            for c in commands:
                echo(c[0].ljust(max_width + 1) + c[1])
            print
        elif mode == 'list_modules':
            echo(u'モジュール一覧')
            for m in interpreter.get_modules():
                echo(m)
        else:
            return self.do_help('')

    def _echo(self, s):
        cout.write(s)

    @catch
    def do_quit(self, line):
        u"""
Usage: quite
Caty インタープリタの終了。
        """
        print '\nbye'
        return True

    @catch
    def do_change(self, line):
        u"""
Usage: change <SITE_NAME>
サイトの変更を行う。
        """
        return self.change_app(line)

    @catch
    def do_reload(self, line):
        u"""
Usage: reload
Alias: l
commands, schemata などの再読み込み
        """
        self.app.reload()
        self.set_prompt()
        self.interpreter = None
        return False

    def _format_doc(self, doc):
        lines = doc.strip('\n').split('\n')
        space = 0
        for i in list(lines[0]):
            if i == ' ':
                space += 1
            else:
                break
        d = []
        for l in lines:
            d.append(l.replace(' '*space, '', 1))
        return '\n'.join(d)

    @catch
    def do_server(self, line):
        u"""
        Usage: server start|stop
        Web サーバの起動・停止を行う
        """
        def usage():
            self._echo(u'使い方: server start [port] または server stop')
        from caty.front.web import build_server
        cmd = line.strip()
        if ' ' in cmd:
            cmd, rest = map(str.strip, line.split(' ', 1))
        else:
            rest = ''
        port = 8000
        if cmd == 'start':
            if self.server is None:
                if rest:
                    from caty.util import try_parse
                    port = try_parse(int, rest) or 8000
                self.server = ServerThread(build_server(self.system, self.debug, port))
                self.server.start()
            else:
                self._echo(u'サーバは既に起動しています')
        elif cmd == 'stop':
            if self.server is not None:
                self.server.stop()
                self.server = None
            else:
                self._echo(u'サーバが起動していません')
        elif cmd == '':
            usage()
        else:
            self._echo(u'未知の引数: %s' % cmd)
            usage()

    do_l = do_reload

    def default(self, line):
        facilities = self.start_pipeline()
        if line == 'EOF':
            print '\nbye'
            return True
        else:
            if line in ('\n', '\r', '\r\n') and self.prompt.startswith('caty'):
                # 単なる空行には何もしない
                return False
            try:
                # 一連のパイプラインが入力しきっていない場合、 None が返る
                try:
                    c = self.interpreter.build(unicode(line.strip(), self.app.sysencoding))
                except NothingTodo:
                    self.set_prompt()
                    return False
                if c:
                    r = c(None)
                    if self.not_void_out(c):
                        o = pp(r)
                        self.stream.write(o + '\n')
                    self.set_prompt()
                else:
                    self.prompt = '> '
            except Exception, e:
                cout.write(traceback)
                cout.write(e)
                self.set_prompt()
            finally:
                self.last_session = lambda :facilities['session']

    def set_prompt(self):
        self.prompt = 'caty:%s> ' % ("" if self.app.name == "_ROOT" else self.app.name)

    def format(self, obj):
        if isinstance(obj, unicode):
            o = '"%s"' % obj
        elif isinstance(obj, dict):
            o = pp(obj)
        elif isinstance(obj, list):
            o = '[%s]' % ', '.join([self.format(v) for v in obj])
        else:
            o = str(obj)
        return o

    def not_void_out(self, command):
        scm = command.out_schema
        if scm.type == 'void':
            return False
        elif scm.type == 'array':
            if all(map(lambda s: s.type == 'void', scm.schema_list)):
                return False
        return True

    def completedefault(self, text, line, bi, ei):
        files = glob.glob(text + '*')
        return list(files)

    def emptyline(self):
        return self.default('\n')

    def exec_files(self, files):
        for f in files:
            self.start_pipeline()
            s = unicode(open(f, 'rb').read(), self.app.encoding)
            try:
                c = self.file_loader.build(s)
                r = c(None)
                if self.not_void_out(c):
                    o = pp(r)
                    self.stream.write(o + '\n')
            except Exception, e:
                cout.write(traceback)
                cout.write(e)
                print e

    def exec_script(self, script):
        self.start_pipeline()
        try:
            c = self.file_loader.build(script)
            r = c(None)
            if self.not_void_out(c):
                o = pp(r)
                self.stream.write(o + '\n')
        except Exception, e:
            cout.write(traceback)
            cout.write(e)
            print e


    def cmdloop(self, *args, **kwds):
        try:
            cmd.Cmd.cmdloop(self, *args, **kwds)
        except KeyboardInterrupt, e:
            self.cleanup()
            raise

    def cleanup(self):
        if self.server is not None:
            self.server.stop()

import sys
try:
    import simplejson as json
except ImportError:
    import json
def exc_wrapper(f):
    def wrapped(*args, **kwds):
        try:
            return f(*args, **kwds)
        except Exception, e:
            cout.write(traceback)
            cout.write(e)
            return -1
    return wrapped

_encoding = get_encoding()
from caty.core.application import System
def getsite(args):
    opts, args = getopt(args, 'e:s:dua:', ['eval=', 'system-encoding=', 'app=', 'unleash-wildcats', 'debug'])
    sitename = 'root'
    wildcat = False
    debug = True
    system_encoding = None
    script = ''
    global _encoding
    _encoding = get_encoding()
    init_writer(_encoding)
    for o, v in opts:
        if o in ('-a', '--app'):
            sitename = v
        elif o in ('-u', '--unleash-wildcats'):
            wildcat = True
        elif o in ('-d', '--debug'):
            debug = True
        elif o in ('-s', '--system-encoding'):
            system_encoding = v
            _encoding = v
            init_writer(_encoding)
        elif o in ('-e', '--eval'):
            script = v
    system = System(_encoding, debug)
    site = system.get_app(sitename)
    return site, wildcat, debug, args, system, script

import threading
class ServerThread(threading.Thread):
    def __init__(self, server):
        threading.Thread.__init__(self)
        self.server = server

    def run(self):
        self.server.main()

    def stop(self):
        self.server.close()

@exc_wrapper
def main(args):
    if sys.platform == 'win32':
        import pyreadline as readline
        from ctypes import windll
    else:
        import readline
    orgdelims = readline.get_completer_delims()
    newdelims = orgdelims.replace('/', '')
    readline.set_completer_delims(newdelims)
    site, wildcat, debug, args, system, script = getsite(args)
    sh = CatyShell(site, wildcat, debug, system)
    if not args and not script:
        if sys.platform == 'win32':
            windll.kernel32.SetConsoleTitleW(u'Caty Console')
        sh.cmdloop()
    else:
        if not script:
            sh.exec_files(args)
        else:
            sh.exec_script(script)
    sh.cleanup()
    return 0

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))

