#coding:utf-8
import sys
import os
from caty.core.application import System
from caty.front.console import CatyShell, get_encoding
from caty.util import init_writer
from wsgiref.simple_server import make_server

def main(args):
    system, is_debug, port, shell = setup(args)
    server = build_server(system, is_debug, port)
    if shell is not None:
        console_thread = ConsoleThread(shell, server)
        console_thread.start()
    server.main()

def build_server(system, is_debug, port=8000):
    server_module_name = system.server_module_name
    exec 'import %s as server_module' % server_module_name
    server = CatyServerFacade(server_module, system, is_debug, port)
    return server

def setup(args):
    from getopt import getopt
    opts, args = getopt(args, 'se:dp:', ['with-shell', 'system-encoding=', 'debug', 'port='])
    debug = False
    system_encoding = None
    use_shell = False
    port = 8000
    _encoding = get_encoding()
    init_writer(_encoding)
    for o, v in opts:
        if o in ('-d', '--debug'):
            debug = True
        elif o in ('-e', '--system-encoding'):
            system_encoding = v
            init_writer(system_encoding)
        elif o in ('-s', '--with-shell'):
            use_shell = True
        elif o in ('-p', '--port'):
            port = int(v)
    system = System(system_encoding, debug)
    return system, debug, port, CatyShell(system.get_app('root'), False, debug, system) if use_shell else None

import threading
class ConsoleThread(threading.Thread):
    def __init__(self, shell, server):
        threading.Thread.__init__(self)
        self.shell = shell
        self.server = server

    def run(self):
        self.shell.cmdloop()
        self.server.close()

    def stop(self):
        self.shell.do_quit()

class CatyServerFacade(object):
    def __init__(self, server_module, system, is_debug, port):
        self.is_debug = is_debug
        server_class = server_module.get_server(system, is_debug)
        handler_class = server_module.get_handler(system, is_debug)
        dispatcher = server_module.get_dispatcher(system, is_debug)
        self.httpd = make_server('', 
                                 port, 
                                 dispatcher, 
                                 server_class, 
                                 handler_class)
        print "Serving on port %d..." % port

    def main(self):
        self.httpd.serve_forever()
   
    def close(self):
        self.httpd.server_close()

if __name__ == '__main__':
    main()

