# coding: utf-8
from topdown import *
class JsonQueryError(Exception): pass

class Accessor(object):
    def __init__(self):
        self.error = False

    def find(self, json):
        raise NotImplementedError

    @property
    def key(self):
        if not self.error:
            return str(self._key())
        else:
            return str('<%s>' % self._key())

    def reset(self):
        self.error = False

    def _key(self):
        raise NotImplementedError

class RootAccessor(Accessor):
    def find(self, json):
        yield json

    def _key(self):
        return '$'

class NameAccessor(Accessor):
    def __init__(self, name):
        Accessor.__init__(self)
        self._name = name

    def find(self, json):
        yield json[self._name]

    def _key(self):
        return self._name if not ' ' in self._name else '"%s"' % self._name

class IndexAccessor(NameAccessor):
    def find(self, json):
        yield json[int(self._name)]

class WildcardNameAccesor(Accessor):
    def find(self, json):
        if not hasattr(json, 'values'):
            for v in json.values():
                yield v
        else:
            raise JsonQueryError(u'引数が JSON オブジェクトではありません')

    def _key(self):
        return '*'

class WildcardIndexAccesor(Accessor):
    def find(self, json):
        if isinstance(json, (list, tuple)):
            for v in json:
                yield v
        else:
            raise JsonQueryError(u'引数がリストではありません')

    def _key(self):
        return '#'

class AccessorPair(Accessor):
    def __init__(self, a, b):
        self._prev = a
        self._next = b

    def find(self, json):
        for j in self._prev.find(json):
            if j is None: 
                yield j
            else:
                try:
                    for o in self._next.find(j):
                        yield o
                except:
                    self._next.error = True
                    raise
    
    def error():
        def get(self):
            return self._next.error

        def set(self, value):
            self._next.error = value

        return get, set
    error = property(*error())

    def reset(self):
        self._next.reset()

    def key(self):
        return '%s.%s' % (self._prev.key, self._next.key)

@as_parser
def query(seq):
    qs = accessor(seq)
    return reduce(lambda a, b: AccessorPair(a, b), qs)

def accessor(seq):
    r = root(seq)
    _ = seq.parse(option('.'))
    if _:
        tokens = split([number, name, quoted_name, o_wild, i_wild], '.')(seq)
        return [r] + tokens
    else:
        return [r]

def root(seq):
    _ = seq.parse('$')
    return RootAccessor()

def name(seq):
    n = seq.parse(Regex(r'[^ "#*.]+'))
    return NameAccessor(n)

def quoted_name(seq):
    _ = seq.parse('"')
    s = seq.parse(until('"'))
    _ = seq.parse('"')
    return NameAccessor(s)

def number(seq):
    n = seq.parse(Regex(r'[0-9]+'))
    return IndexAccessor(n)

def o_wild(seq):
    _ = seq.parse('*')
    return WildcardNameAccesor()

def i_wild(seq):
    _ = seq.parse('#')
    return WildcardIndexAccesor()

