#coding: utf-8
from topdown import *
import caty.jsontools as json
from caty.util import bind2nd
from decimal import Decimal
from caty.jsontools import stdjson

def load(fo):
    s = fo.read()
    return loads(s)

def loads(s):
    cs = CharSeq(s, hook=remove_comment, auto_remove_ws=True)
    r = cs.parse(parsers)
    if not cs.eof:
        raise Exception(cs.text)
    return r

def dumps(obj):
    return json.pp(obj)

def dump(obj, fo):
    fo.write(dumps(obj))

@try_
def string(seq):
    def series_of_escape(s):
        import itertools
        return len(list(itertools.takewhile(lambda c: c=='\\', reversed(s))))
    try:
        seq.ignore_hook = True
        st = [seq.parse('"')]
        s = seq.parse(until('"'))
        while True:
            if series_of_escape(s) % 2 == 0:
                st.append(s)
                break
            else:
                st.append(s)
                s = seq.parse(Regex(r'"[^"]*'))
        st.append(seq.parse('"'))
        return stdjson.loads(''.join(st))
    except EndOfBuffer, e:
        raise ParseFailed(seq, string)
    finally:
        seq.ignore_hook = False

def multiline_string(seq):
    try:
        seq.ignore_hook = True
        _ = seq.parse("'''")
        s = seq.parse(until("'''"))
        seq.ignore_hook = False
        _ = seq.parse("'''")
        return stdjson.loads('"%s"' % s.replace('\\', '\\\\').replace('"', '\\"').replace('\n', '\\n').replace('\r', '\\r'))
    finally:
        seq.ignore_hook = False

def boolean(seq, convert=False):
    b = seq.parse(['true', 'false'])
    if convert:
        return True if b == 'true' else False
    else:
        return b

def integer(seq, convert=False):
    i = seq.parse([
                   Regex(r'-?[1-9][0-9]+'),
                   Regex(r'-?[0-9]'),
                 ])
    if convert:
        return int(i)
    else:
        return i

def number(seq, convert=False):
    n = seq.parse([
                   Regex(r'-?[1-9][0-9]+\.[0-9]+([eE][-+]?[0-9]+)?'), 
                   Regex(r'-?[0-9]\.[0-9]+([eE][-+]?[0-9]+)?'), 
                   Regex(r'-?[1-9][0-9]+[eE][-+]?[0-9]+'),
                   Regex(r'-?[0-9][eE][-+]?[0-9]+'),
                   ])
    if convert:
        return Decimal(n)
    else:
        return n

def null(seq, convert=False):
    n = seq.parse('null')
    if convert:
        return None
    else:
        return n

def comma(seq):
    seq.parse(',')

def obj(seq):
    seq.parse('{')
    items = option(split(item, comma, True), {})(seq)
    seq.parse('}')
    o = {}
    for k ,v in items:
        o[k] = v
    return o

def item(seq):
    k = seq.parse(string)
    seq.parse(':')
    v = seq.parse(parsers)
    return k, v

def array(seq):
    seq.parse('[')
    items = option(split(parsers, comma, True), [])(seq)
    seq.parse(']')
    return items

def tag(seq):
    _ = seq.parse('@')
    name = seq.parse([string, Regex(r'[-0-9a-zA-Z_]+')])
    value = seq.parse(parsers)
    return json.TaggedValue(name, value)

parsers = [
    tag,
    bind2nd(number, True), 
    bind2nd(integer, True), 
    string, 
    multiline_string, 
    bind2nd(boolean, True), 
    array, 
    obj]

def remove_comment(seq):
    seq.parse(singleline_comment)
    seq.parse(multiline_comment)
    seq.parse(skip_ws)

def singleline_comment(seq):
    if seq.parse(option('//', None)) == None: return
    seq.parse([until('\n'), until('\r')])

def multiline_comment(seq):
    if seq.parse(option('/*', None)) == None: return
    seq.parse(until('*/'))
    seq.parse('*/')

