#coding: utf-8
u"""Smarty 構文サブセットのパーサ。
"""

from topdown import *
from caty.template.core.st import *
from caty.util import bind2nd

@as_parser
def smarty_parser(seq):
    t = smarty_template(seq)
    if not seq.eof:
        raise ParseFailed(seq, None)
    return t

def smarty_template(seq):
    ts = seq.parse(many(term))
    return Template(ts)

def term(seq):
    s = seq.parse([literal, statement, comment])
    return s

def literal(seq):
    s = seq.parse(until('{'))
    if not s:
        raise ParseFailed(seq, literal)
    escape = seq.parse(option('{literal}'))
    if escape is None:
        return String(s)
    else:
        s2 = seq.parse(until('{/literal}'))
        seq.parse('{/literal}')
        return String(''.join([s, s2]))

def comment(seq):
    seq.parse('{*')
    seq.parse(until('*}'))
    seq.parse('*}')
    return Null()

def statement(seq):
    s = seq.parse([if_, for_, include, try_(var)])
    return s

def var(seq):
    ob = seq.parse('{')
    ws = seq.parse(option(skip_ws))
    load = varload(seq)
    ws = seq.parse(option(skip_ws))
    cb = seq.parse('}')
    return Template([load, VarDisp()])

def varload(seq):
    d = seq.parse('$')
    v = varref(seq)
    filters = many(filter_call)(seq)
    if not filters:
        return VarLoad(v, True)
    else:
        return Template([VarLoad(v, True)] + filters)

def varref(seq):
    r = [varname(seq)]
    while True:
        v = option('.')(seq)
        if not v:
            break
        else:
            r.append(v+varname(seq))
    return ''.join(r)

def varname(seq):
    name = seq.parse(Regex(r'[a-zA-Z_][a-zA-Z_0-9]*'))
    idx = seq.parse(many(index))
    return name + '.' + ('.'.join(idx)) if idx != [] else name

def index(seq):
    # [] を削除した上で値を戻す
    r = seq.parse([Regex(r'\[[0-9]+\]'), Regex(r'\[".+?"\]')])
    return  r[1:-1].strip('"')

def filter_call(seq):
    skip_ws(seq)
    seq.parse('|')
    skip_ws(seq)
    name = seq.parse(Regex(r'[a-zA-Z_][a-zA-Z_0-9]*'))
    skip_ws(seq)
    args = many(argument)(seq)
    return FilterCall(name, args)

def argument(seq):
    def integer(s):
        return int(s.parse(Regex(r'-?[0-9]+')))

    def number(s):
        return float(s.parse(Regex(r'-?[0-9]+\.[0-9]+')))

    def boolean(s):
        return bool(seq.parse(['true', 'false']).toupper())

    def string(s):
        s.parse('"')
        r = s.parse(until('"'))
        s.parse('"')
        return u'%s' % r

    skip_ws(seq)
    seq.parse(':')
    skip_ws(seq)
    return seq.parse([integer, number, boolean, string])

def include(seq):
    i = seq.parse('{include')
    ws = seq.parse(skip_ws)
    f = seq.parse('file')
    ws = seq.parse(option(skip_ws))
    e = seq.parse('=')
    ws = seq.parse(option(skip_ws))
    q = seq.parse('"')
    name = seq.parse(until('"'))
    q = seq.parse('"')
    ws = seq.parse(option(skip_ws))
    c = seq.parse(option('context'))
    if c:
        e = seq.parse('=')
        ws = seq.parse(option(skip_ws))
        q = seq.parse('"')
        cn = seq.parse(until('"'))
        q = seq.parse('"')
    else:
        cn = u'_CONTEXT'
    ws = seq.parse(option(skip_ws))
    cb = seq.parse('}')
    return Include(name, cn)

def if_(seq):
    _ = seq.parse('{if')
    ws = seq.parse(skip_ws)
    varnode = varload(seq)
    ws = seq.parse(skip_ws)
    _ = seq.parse('}')
    subtempl = smarty_template(seq)
    elifnodes = many(elif_)(seq) or Null()
    elsenode = option(else_, Null())(seq)
    _ = seq.parse('{/if}')
    return If(varnode, subtempl, elifnodes, elsenode)

def elif_(seq):
    _ = seq.parse('{elif')
    ws = seq.parse(skip_ws)
    varnode = var(seq)
    ws = seq.parse(skip_ws)
    _ = seq.parse('}')
    subtempl = smarty_template(seq)
    return Elif(varnode, subtempl)

def else_(seq):
    _ = seq.parse('{else}')
    subtempl = smarty_template(seq)
    return Else(subtempl)

def for_(seq):
    _ = seq.parse('{foreach')
    ws = seq.parse(skip_ws)
    attr = loop_attr(seq)
    loopitem = attr.get('item')
    loopkey = attr.get('key', 'key')
    loop_name = attr.get('name', 'default')
    varnode = attr.get('var')
    ws = seq.parse(skip_ws)
    _ = seq.parse('}')
    subtempl = smarty_template(seq)
    elsetempl = option(for_else, Null())(seq)
    _ = seq.parse('{/foreach}')
    index_name = "smarty.foreach.%s.index" % loop_name
    iteration_name = "smarty.foreach.%s.iteration" % loop_name
    counter_name = "smarty.foreach.%s.total" % loop_name
    first = "smarty.foreach.%s.first" % loop_name
    last = "smarty.foreach.%s.last" % loop_name
    return For(loopitem, 
               varnode, 
               loopkey, 
               index_name, 
               iteration_name, 
               counter_name,
               first,
               last,
               subtempl,
               elsetempl)

def loop_attr(seq):
    return dict(seq.parse(many1([loop_item, loop_key, loop_name, loop_var])))

def loop_var(seq):
    skip_ws(seq)
    seq.parse('from')
    skip_ws(seq)
    seq.parse('=')
    skip_ws(seq)
    load = varload(seq)
    return ('var', load)

def _loop_attr(seq, t):
    skip_ws(seq)
    seq.parse(t)
    skip_ws(seq)
    seq.parse('=')
    item = seq.parse(Regex(r'[a-zA-Z_][a-zA-Z0-9_]*'))
    return (t, item)

loop_item = bind2nd(_loop_attr, 'item')
loop_key = bind2nd(_loop_attr, 'key')
loop_name = bind2nd(_loop_attr, 'name')

def for_else(seq):
    seq.parse('else}')
    return smarty_template(seq)

