#coding: utf-8
from __future__ import with_statement
from threading import RLock
from caty.util.collection import *
from caty.util.cache import memoize
u"""共通的に使われるユーティリティ群。
"""
def try_parse(type, s):
    u"""s を type に変換する。
    失敗時には None を返す。
    """
    try:
        return type(str(s))
    except:
        return None

def bind1st(f, a):
    def bound(b):
        return f(a, b)
    return bound

def bind2nd(f, b):
    def bound(a):
        return f(a, b)
    return bound

def error_to_ustr(e):
    import sys
    v = float(sys.version.split(' ')[0].rsplit('.', 1)[0])
    if v <= 2.5:
        return e.message if isinstance(e.message, unicode) else unicode(e.message, 'utf-8')
    else:
        return unicode(e)

def get_message(e, encoding):
    import sys
    v = float(sys.version.split(' ')[0].rsplit('.', 1)[0])
    if v <= 2.5:
        return e.message if not isinstance(e.message, unicode) else e.message.encode(encoding)
    else:
        try:
            if hasattr(e, 'filename'):
                return _encode(e.strerror, encoding) + ':' + _encode(e.filename, encoding)
            return _encode(e.strerror, encoding)
        except:
            try:
                return unicode(e).encode(encoding)
            except:
                return repr(e)
    
def _encode(s, encoding):
    if isinstance(s, unicode):
        return s.encode(encoding)
    else:
        return s

def to_unicode(val):
    if isinstance(val, unicode):
        return val
    elif isinstance(val, bool):
        return unicode(str(val)).lower()
    else:
        if val is not None:
            return unicode(str(val))

def get_charset(obj):
    import re
    matcher = re.compile('charset=(.+)')
    matched = matcher.search(obj['header']['content-type'])
    if matched:
        return matched.group(1)
    else:
        return None

def escape_html(s):
    return s.replace('&', '&amp;').replace('"', '&quot;').replace('<', '&lt;').replace('>', '&gt;').replace('\'', '&#39;')

import sys, traceback
class ConsoleWriter(object):
    def __init__(self):
        self.encoding = None # 端末エンコーディングが指定される
        self.stream = sys.stdout

    def set_encoding(self, encoding):
        self.encoding = encoding

    def write(self, arg):
        assert self.encoding is not None
        assert isinstance(arg, (unicode, Exception)) or arg is traceback
        if isinstance(arg, unicode):
            self.stream.write(arg.encode(self.encoding))
        elif isinstance(arg, Exception):
            self.stream.write(get_message(arg, self.encoding))
        else:
            try:
                tb = unicode(traceback.format_exc(), self.encoding)
            except:
                tb = unicode(traceback.format_exc(), self.encoding)
            self.stream.write(tb.encode(self.encoding))
        self.stream.write('\n')

class DebugWriter(ConsoleWriter):
    def write(self, arg):
        assert isinstance(arg, (unicode, Exception))
        self.stream.write('DEBUG: ')
        ConsoleWriter.write(self, arg)
        self.stream.write('\n')

cout = ConsoleWriter()
debug = DebugWriter()

def init_writer(encoding):
    cout.set_encoding(encoding)
    debug.set_encoding(encoding)
    return cout, debug

def error_wrapper(f):
    def _(*args, **kwds):
        try:
            return f(*args, **kwds)
        except Exception, e:
            print traceback.format_exc()
            raise
    return _


def deprecated(msg):
    def _deprecated(f):
        def func(*args, **kwds):
            traceback.print_stack()
            cout.write(u'DEPRECATED: ' + repr(f))
            cout.write('            ' + msg)
            return f(*args, **kwds)
        return func
    return _deprecated

