from topdown import *
from creole.tree import Element
import string
halfwidth_char = set(string.printable.replace(' ', '').replace('\t', '').replace('\r', '').replace('\n', '').replace('\x0b', '').replace('\x0c', ''))

class WikiParser(Parser):
    def __init__(self, element_factory=None):
        self._element_factory = element_factory if element_factory else Element

    def create_element(self, *args, **kwds):
        return self._element_factory(*args, **kwds)

class BlockParser(WikiParser):
    def __init__(self, syntax, factory=None):
        self.language = syntax.language
        self.inline = syntax.inline
        self.syntax= syntax
        WikiParser.__init__(self, factory)

    def _format(self, text):
        text = text.strip(' \r\t\n\r')
        if self.language == 'jp':
            lines = text.split('\n')
            r = []
            last = ''
            for l in lines:
                if l:
                    if last in halfwidth_char or l[0] in halfwidth_char:
                        r.append('\n')
                    r.append(l)
                    last = l[-1]
            return ''.join(r).strip('\n')
        return text

class InlineParser(WikiParser):

    def _enter(self, seq):
        t = []
        while not seq.eof:
            s = seq.parse(until([self.end, '\n']))
            if s.endswith('~') and seq.peek(option(self.end)):
                t.append(s[:-1])
                t.append(seq.parse(self.end))
            else:
                seq.parse(option(self.end))
                t.append(s)
                break
        text = ''.join(t)
        return text
